@@ -9,7 +9,6 @@ for which a new license (GPL+exception) is in place.
 #include <QSplitter>
 #include <QMenuBar>
 #include <QMenu>
-#include <QProcess>
 #include <QTime>
 
 #include <QInputDialog>
@@ -18,6 +17,7 @@ for which a new license (GPL+exception) is in place.
 
 #include <QSqlDatabase>
 #include <QSqlError>
+#include <QSqlQuery>
 
 #include <QCoreApplication>
 #include <QCloseEvent>
@@ -28,6 +28,7 @@ for which a new license (GPL+exception) is in place.
 #include <QDir>
 
 #include "litemanwindow.h"
+#include "preferences.h"
 #include "preferencesdialog.h"
 #include "queryeditordialog.h"
 #include "createtabledialog.h"
@@ -37,7 +38,7 @@ for which a new license (GPL+exception) is in place.
 #include "altertabledialog.h"
 #include "altertriggerdialog.h"
 #include "dataviewer.h"
-#include "tabletree.h"
+#include "schemabrowser.h"
 #include "database.h"
 #include "sqleditor.h"
 #include "sqlmodels.h"
@@ -46,32 +47,77 @@ for which a new license (GPL+exception) is in place.
 #include "analyzedialog.h"
 #include "vacuumdialog.h"
 #include "helpbrowser.h"
-#include "sqlparser.h"
+#include "importtabledialog.h"
+#include "sqliteprocess.h"
+#include "populatordialog.h"
+#include "utils.h"
+
+#include <QProcess>
+#include <QtDebug>
 
 
 LiteManWindow::LiteManWindow(const QString & fileToOpen)
 	: QMainWindow(),
 	m_mainDbPath(""),
 	m_appName("Sqliteman"),
+	m_sqliteBinAvailable(true),
 	helpBrowser(0)
 {
+	// test for sqlite3 binary
+	qDebug() << "Checking for " << SQLITE_BINARY << ":";
+	if (QProcess::execute(SQLITE_BINARY, QStringList() << "-version") != 0)
+	{
+		m_sqliteBinAvailable = false;
+		QMessageBox::warning(this, m_appName,
+							 tr("Sqlite3 executable '%1' is not found in your path. Some features will be disabled.")
+									 .arg(SQLITE_BINARY));
+		qDebug() << "Sqlite3 executable '%1' is not found in your path. Some features will be disabled.";
+	}
+	qDebug() << "Checking for Qt version: " << qVersion();
+#if QT_VERSION < 0x040300
+	if (Preferences::instance()->checkQtVersion())
+	{
+		QMessageBox::warning(this, m_appName,
+						 tr("Sqliteman is using Qt %1. Some features will be disabled.")
+							.arg(qVersion()));
+		qDebug() << "Sqliteman is using Qt %1. Some features will be disabled.";
+	}
+#endif
+
 	recentDocs.clear();
 	attachedDb.clear();
 	initUI();
 	initActions();
 	initMenus();
-// 	initStatusBar();
+
 	statusBar();
+	m_sqliteVersionLabel = new QLabel(this);
+	statusBar()->addPermanentWidget(m_sqliteVersionLabel);
+
 	readSettings();
 
 	// Check command line
-	if (!fileToOpen.isNull())
+	qDebug() << "Initial DB: " << fileToOpen;
+	if (!fileToOpen.isNull() && !fileToOpen.isEmpty())
 		open(fileToOpen);
 }
 
+LiteManWindow::~LiteManWindow()
+{
+	Preferences::deleteInstance();
+}
+
 void LiteManWindow::closeEvent(QCloseEvent * e)
 {
 	sqlEditor->saveOnExit();
+	writeSettings();
+
+	// check for uncommited transaction in the DB
+	if (!dataViewer->setTableModel(new QSqlQueryModel()))
+	{
+		e->ignore();
+		return;
+	}
 
 	QMapIterator<QString, QString> i(attachedDb);
 	while (i.hasNext())
@@ -81,7 +127,10 @@ void LiteManWindow::closeEvent(QCloseEvent * e)
 		QSqlDatabase::database(i.value()).close();
 	}
 
-	writeSettings();
+	// It has to go after writeSettings()!
+	foreach (QWidget *widget, QApplication::topLevelWidgets())
+		widget->close();
+
 	e->accept();
 }
 
@@ -92,8 +141,8 @@ void LiteManWindow::initUI()
 	splitter = new QSplitter(this);
 	splitterSql = new QSplitter(Qt::Vertical, this);
 
-	tableTree = new TableTree();
-	dataViewer = new DataViewer();
+	schemaBrowser = new SchemaBrowser(this);
+	dataViewer = new DataViewer(this);
 	sqlEditor = new SqlEditor(this);
 
 	splitterSql->addWidget(sqlEditor);
@@ -101,7 +150,7 @@ void LiteManWindow::initUI()
 	splitterSql->setCollapsible(0, false);
 	splitterSql->setCollapsible(1, false);
 
-	splitter->addWidget(tableTree);
+	splitter->addWidget(schemaBrowser);
 	splitter->addWidget(splitterSql);
 	splitter->setCollapsible(0, false);
 	splitter->setCollapsible(1, false);
@@ -109,32 +158,41 @@ void LiteManWindow::initUI()
 	setCentralWidget(splitter);
 
 	// Disable the UI, as long as there is no open database
-	tableTree->setEnabled(false);
+	schemaBrowser->setEnabled(false);
 	dataViewer->setEnabled(false);
 	sqlEditor->setEnabled(false);
 	sqlEditor->hide();
 
-	connect(tableTree, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
+	connect(schemaBrowser->tableTree, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
 		this, SLOT(treeItemActivated(QTreeWidgetItem *, int)));
 
-	connect(tableTree, SIGNAL(customContextMenuRequested(const QPoint &)),
+	connect(schemaBrowser->tableTree, SIGNAL(customContextMenuRequested(const QPoint &)),
 		this, SLOT(treeContextMenuOpened(const QPoint &)));
-	connect(tableTree, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
+	connect(schemaBrowser->tableTree, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
 			this, SLOT(tableTree_currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)));
 
 	// sql editor
-	connect(sqlEditor, SIGNAL(showSqlResult(QString)), this, SLOT(execSql(QString)));
+	connect(sqlEditor, SIGNAL(showSqlResult(QString)),
+			this, SLOT(execSql(QString)));
+	connect(sqlEditor, SIGNAL(sqlScriptStart()),
+			dataViewer, SLOT(sqlScriptStart()));
+	connect(sqlEditor, SIGNAL(showSqlScriptResult(QString)),
+			dataViewer, SLOT(showSqlScriptResult(QString)));
 	connect(sqlEditor, SIGNAL(rebuildViewTree(QString, QString)),
-			tableTree, SLOT(buildViewTree(QString,QString)));
+			schemaBrowser->tableTree, SLOT(buildViewTree(QString,QString)));
+	connect(sqlEditor, SIGNAL(buildTree()),
+			schemaBrowser->tableTree, SLOT(buildTree()));
 }
 
 void LiteManWindow::initActions()
 {
-	newAct = new QAction(tr("&New..."), this);
+	newAct = new QAction(Utils::getIcon("document-new.png"),
+						 tr("&New..."), this);
 	newAct->setShortcut(tr("Ctrl+N"));
 	connect(newAct, SIGNAL(triggered()), this, SLOT(newDB()));
 
-	openAct = new QAction(tr("&Open..."), this);
+	openAct = new QAction(Utils::getIcon("document-open.png"),
+						  tr("&Open..."), this);
 	openAct->setShortcut(tr("Ctrl+O"));
 	connect(openAct, SIGNAL(triggered()), this, SLOT(open()));
 
@@ -145,7 +203,7 @@ void LiteManWindow::initActions()
 	preferencesAct = new QAction(tr("&Preferences..."), this);
 	connect(preferencesAct, SIGNAL(triggered()), this, SLOT(preferences()));
 
-	exitAct = new QAction(tr("E&xit"), this);
+	exitAct = new QAction(Utils::getIcon("close.png"), tr("E&xit"), this);
 	exitAct->setShortcut(tr("Ctrl+Q"));
 	connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));
 
@@ -178,8 +236,10 @@ void LiteManWindow::initActions()
 
 	dumpDatabaseAct = new QAction(tr("&Dump Database..."), this);
 	connect(dumpDatabaseAct, SIGNAL(triggered()), this, SLOT(dumpDatabase()));
+	dumpDatabaseAct->setEnabled(m_sqliteBinAvailable);
 
-	createTableAct = new QAction(tr("&Create Table..."), this);
+	createTableAct = new QAction(Utils::getIcon("table.png"),
+								 tr("&Create Table..."), this);
 	createTableAct->setShortcut(tr("Ctrl+T"));
 	connect(createTableAct, SIGNAL(triggered()), this, SLOT(createTable()));
 
@@ -190,7 +250,14 @@ void LiteManWindow::initActions()
 	alterTableAct->setShortcut(tr("Ctrl+A"));
 	connect(alterTableAct, SIGNAL(triggered()), this, SLOT(alterTable()));
 
-	createViewAct = new QAction(tr("Create &View..."), this);
+	renameTableAct = new QAction(tr("&Rename Table..."), this);
+	connect(renameTableAct, SIGNAL(triggered()), this, SLOT(renameTable()));
+
+	populateTableAct = new QAction(tr("&Populate Table..."), this);
+	connect(populateTableAct, SIGNAL(triggered()), this, SLOT(populateTable()));
+
+	createViewAct = new QAction(Utils::getIcon("view.png"),
+								tr("Create &View..."), this);
 	createViewAct->setShortcut(tr("Ctrl+G"));
 	connect(createViewAct, SIGNAL(triggered()), this, SLOT(createView()));
 
@@ -200,16 +267,21 @@ void LiteManWindow::initActions()
 	alterViewAct = new QAction(tr("&Alter View..."), this);
 	connect(alterViewAct, SIGNAL(triggered()), this, SLOT(alterView()));
 
-	createIndexAct = new QAction(tr("&Create Index..."), this);
+	createIndexAct = new QAction(Utils::getIcon("index.png"),
+								 tr("&Create Index..."), this);
 	connect(createIndexAct, SIGNAL(triggered()), this, SLOT(createIndex()));
 
 	dropIndexAct = new QAction(tr("&Drop Index"), this);
 	connect(dropIndexAct, SIGNAL(triggered()), this, SLOT(dropIndex()));
 
 	describeTableAct = new QAction(tr("D&escribe Table"), this);
-	connect(describeTableAct, SIGNAL(triggered()), this, SLOT(describeTable()));
+	connect(describeTableAct, SIGNAL(triggered()), this, SLOT(describeObject()/*describeTable()*/));
 
-	createTriggerAct = new QAction(tr("&Create Trigger..."), this);
+	importTableAct = new QAction(tr("&Import Table Data..."), this);
+	connect(importTableAct, SIGNAL(triggered()), this, SLOT(importTable()));
+
+	createTriggerAct = new QAction(Utils::getIcon("trigger.png"),
+								   tr("&Create Trigger..."), this);
 	connect(createTriggerAct, SIGNAL(triggered()), this, SLOT(createTrigger()));
 
 	alterTriggerAct = new QAction(tr("&Alter Trigger..."), this);
@@ -219,13 +291,13 @@ void LiteManWindow::initActions()
 	connect(dropTriggerAct, SIGNAL(triggered()), this, SLOT(dropTrigger()));
 
 	describeTriggerAct = new QAction(tr("D&escribe Trigger"), this);
-	connect(describeTriggerAct, SIGNAL(triggered()), this, SLOT(describeTrigger()));
+	connect(describeTriggerAct, SIGNAL(triggered()), this, SLOT(describeObject()/*describeTrigger()*/));
 
 	describeViewAct = new QAction(tr("D&escribe View"), this);
-	connect(describeViewAct, SIGNAL(triggered()), this, SLOT(describeView()));
+	connect(describeViewAct, SIGNAL(triggered()), this, SLOT(describeObject()/*describeView()*/));
 
 	describeIndexAct = new QAction(tr("D&escribe Index"), this);
-	connect(describeIndexAct, SIGNAL(triggered()), this, SLOT(describeIndex()));
+	connect(describeIndexAct, SIGNAL(triggered()), this, SLOT(describeObject()/*describeIndex()*/));
 
 	reindexAct = new QAction(tr("&Reindex"), this);
 	connect(reindexAct, SIGNAL(triggered()), this, SLOT(reindex()));
@@ -243,7 +315,7 @@ void LiteManWindow::initActions()
 	connect(detachAct, SIGNAL(triggered()), this, SLOT(detachDatabase()));
 
 	refreshTreeAct = new QAction(tr("&Refresh Object Tree"), this);
-	connect(refreshTreeAct, SIGNAL(triggered()), tableTree, SLOT(buildTree()));
+	connect(refreshTreeAct, SIGNAL(triggered()), schemaBrowser->tableTree, SLOT(buildTree()));
 
 	consTriggAct = new QAction(tr("&Constraint Triggers..."), this);
 	connect(consTriggAct, SIGNAL(triggered()), this, SLOT(constraintTriggers()));
@@ -273,6 +345,7 @@ void LiteManWindow::initMenus()
 	databaseMenu->addSeparator();
 	databaseMenu->addAction(exportSchemaAct);
 	databaseMenu->addAction(dumpDatabaseAct);
+	databaseMenu->addAction(importTableAct);
 
 	adminMenu = menuBar()->addMenu(tr("&System"));
 	adminMenu->addAction(analyzeAct);
@@ -308,7 +381,7 @@ void LiteManWindow::removeRecent(QString fn)
 void LiteManWindow::rebuildRecentFileMenu()
 {
 	recentFilesMenu->clear();
-	uint max = qMin(PreferencesDialog::recentlyUsedCount(), recentDocs.count());
+	uint max = qMin(Preferences::instance()->recentlyUsedCount(), recentDocs.count());
 	QFile fi;
 	QString accel("&");
 
@@ -343,12 +416,12 @@ void LiteManWindow::readSettings()
 	dataViewer->restoreSplitter(settings.value("dataviewer/splitter").toByteArray());
 
 	sqlEditor->setVisible(settings.value("sqleditor/show", true).toBool());
-	tableTree->setVisible(settings.value("objectbrowser/show", true).toBool());
+	schemaBrowser->setVisible(settings.value("objectbrowser/show", true).toBool());
 	objectBrowserAct->setChecked(settings.value("objectbrowser/show", false).toBool());
 	execSqlAct->setChecked(settings.value("sqleditor/show", false).toBool());
 
 	QString fn(settings.value("sqleditor/filename", QString()).toString());
-	if (!fn.isNull() && !fn.isEmpty())
+	if (!fn.isNull() && !fn.isEmpty() && Preferences::instance()->openLastSqlFile())
 	   sqlEditor->setFileName(fn);
 
 	recentDocs = settings.value("recentDocs/files").toStringList();
@@ -362,12 +435,14 @@ void LiteManWindow::writeSettings()
 	settings.setValue("window/geometry", saveGeometry());
 	settings.setValue("window/size", size());
 	settings.setValue("window/splitter", splitter->saveState());
-	settings.setValue("objectbrowser/show", tableTree->isVisible());
+	settings.setValue("objectbrowser/show", schemaBrowser->isVisible());
 	settings.setValue("sqleditor/show", sqlEditor->isVisible());
 	settings.setValue("sqleditor/splitter", splitterSql->saveState());
 	settings.setValue("sqleditor/filename", sqlEditor->fileName());
 	settings.setValue("dataviewer/splitter", dataViewer->saveSplitter());
 	settings.setValue("recentDocs/files", recentDocs);
+	// last open database
+	settings.setValue("lastDatabase", QSqlDatabase::database(SESSION_NAME).databaseName());
 }
 
 void LiteManWindow::newDB()
@@ -394,9 +469,9 @@ void LiteManWindow::open(const QString & file)
 		fileName = file;
 	else
 		fileName = QFileDialog::getOpenFileName(this,
-			                    tr("Open Database"),
-			                    QDir::currentPath(),
-			                    tr("SQLite database (*)"));
+								tr("Open Database"),
+								QDir::currentPath(),
+								tr("SQLite database (*)"));
 
 	if(fileName.isNull())
 		return;
@@ -405,37 +480,77 @@ void LiteManWindow::open(const QString & file)
 
 void LiteManWindow::openDatabase(const QString & fileName)
 {
-	QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE", SESSION_NAME);
+	bool isOpened = false;
+
+	QSqlDatabase db = QSqlDatabase::database(SESSION_NAME);
+	if (db.isValid())
+	{
+		db.close();
+		QSqlDatabase::removeDatabase(SESSION_NAME);
+	}
+	db = QSqlDatabase::addDatabase("QSQLITE", SESSION_NAME);
 	db.setDatabaseName(fileName);
 
+	QString msg = tr("Unable to open or create file %1. It is probably not a database").arg(QFileInfo(fileName).fileName());
 	if(!db.open())
 	{
-		QString msg = tr("Unable to open or create file %1. It is probably not a database").arg(QFileInfo(fileName).fileName());
 		QMessageBox::warning(this, m_appName, msg);
 		return;
 	}
+	/* Qt database open() (exactly the sqlite API sqlite3_open16())
+	   method does not check if it is really database. So the dummy
+	   select statement should perform a real "is it a database" check
+	   for us. */
+	QSqlQuery q("select 1 from sqlite_master where 1=2", db);
+	if (!q.exec())
+	{
+		QMessageBox::warning(this, m_appName, msg);
+		return;
+	}
+	else
+	{
+		isOpened = true;
 
-	attachedDb.clear();
-	attachedDb["main"] = SESSION_NAME;
+		// check for sqlite library version
+		QString ver;
+		if (q.exec("select sqlite_version(*);"))
+		{
+			if(!q.lastError().isValid())
+			{
+				q.next();
+				ver = q.value(0).toString();
+			}
+			else
+				ver = "n/a";
+		}
+		else
+			ver = "n/a";
+		m_sqliteVersionLabel->setText("Sqlite: " + ver);
 
-	QFileInfo fi(fileName);
-	QDir::setCurrent(fi.absolutePath());
-	m_mainDbPath = QDir::toNativeSeparators(QDir::currentPath() + "/" + fi.fileName());
+		attachedDb.clear();
+		attachedDb["main"] = SESSION_NAME;
+	
+		QFileInfo fi(fileName);
+		QDir::setCurrent(fi.absolutePath());
+		m_mainDbPath = QDir::toNativeSeparators(QDir::currentPath() + "/" + fi.fileName());
+	
+		updateRecent(fileName);
+	
+		// Build tree & clean model
+		schemaBrowser->tableTree->buildTree();
+		schemaBrowser->buildPragmasTree();
+		dataViewer->setTableModel(new QSqlQueryModel(), false);
+	
+		// Update the title
+		setWindowTitle(QString("%1 - %2").arg(fi.fileName()).arg(m_appName));
+	}
 
-	updateRecent(fileName);
-
-	// Build tree & clean model
-	tableTree->buildTree();
-	dataViewer->setTableModel(new QSqlQueryModel());
-
-	// Update the title
-	setWindowTitle(QString("%1 - %2").arg(fi.fileName()).arg(m_appName));
-
-	// Enable UI (Only the tree, the data viewer will be enabled upon demand)
-	tableTree->setEnabled(true);
-	databaseMenu->setEnabled(true);
-	adminMenu->setEnabled(true);
-	sqlEditor->setEnabled(true);
+	// Enable UI
+	schemaBrowser->setEnabled(isOpened);
+	databaseMenu->setEnabled(isOpened);
+	adminMenu->setEnabled(isOpened);
+	sqlEditor->setEnabled(isOpened);
+	dataViewer->setEnabled(isOpened);
 }
 
 void LiteManWindow::openRecent()
@@ -467,13 +582,14 @@ void LiteManWindow::help()
 
 void LiteManWindow::buildQuery()
 {
-	QueryEditorDialog dlg(QueryEditorDialog::BuildQuery, this);
+	QueryEditorDialog dlg(this);
 
-	dlg.resize(500, 400);
+// 	dlg.resize(500, 400);
 	int ret = dlg.exec();
 
 	if(ret == QDialog::Accepted)
-		runQuery(dlg.statement());
+		/*runQuery*/
+		execSql(dlg.statement());
 }
 
 void LiteManWindow::handleSqlEditor()
@@ -484,8 +600,8 @@ void LiteManWindow::handleSqlEditor()
 
 void LiteManWindow::handleObjectBrowser()
 {
-	tableTree->setVisible(!tableTree->isVisible());
-	objectBrowserAct->setChecked(tableTree->isVisible());
+	schemaBrowser->setVisible(!schemaBrowser->isVisible());
+	objectBrowserAct->setChecked(schemaBrowser->isVisible());
 }
 
 void LiteManWindow::execSql(QString query)
@@ -496,7 +612,9 @@ void LiteManWindow::execSql(QString query)
 		return;
 	}
 
+	dataViewer->freeResources();
 	sqlEditor->setStatusMessage();
+
 	QTime time;
 	time.start();
 
@@ -504,23 +622,24 @@ void LiteManWindow::execSql(QString query)
 	SqlQueryModel * model = new SqlQueryModel(this);
 	model->setQuery(query, QSqlDatabase::database(SESSION_NAME));
 
-	if (!dataViewer->setTableModel(model))
+	if (!dataViewer->setTableModel(model, false))
 		return;
 
-	dataViewer->setEnabled(true);
-	dataViewer->showButtons(false);
-	dataViewer->showStatusText(true);
-
 	sqlEditor->setStatusMessage(tr("Duration: %1 seconds").arg(time.elapsed() / 1000.0));
 	
 	// Check For Error in the SQL
 	if(model->lastError().isValid())
-		dataViewer->setStatusText(tr("Query Error: %1\n\n%2").arg(model->lastError().databaseText()).arg(query));
+		dataViewer->setStatusText(tr("Query Error: %1\n\n%2").arg(model->lastError().text()).arg(query));
 	else
 	{
-		dataViewer->setStatusText(tr("Query OK\nRow(s) returned: %1\n\n%2").arg(model->rowCount()).arg(query));
-		if (SqlEditorTools::SqlParser::updateTree(query))
-			tableTree->buildTree();
+		QString cached;
+		if (model->canFetchMore())
+			cached = DataViewer::canFetchMore();
+        else
+            cached = "";
+		dataViewer->setStatusText(tr("Query OK\nRow(s) returned: %1 %2\n%3").arg(model->rowCount()).arg(cached).arg(query));
+		if (Utils::updateObjectTree(query))
+			schemaBrowser->tableTree->buildTree();
 	}
 }
 
@@ -541,33 +660,20 @@ void LiteManWindow::dumpDatabase()
 	if (fileName.isNull())
 		return;
 
-	QProcess dump;
+	SqliteProcess dump(this);
 	dump.setStandardOutputFile(fileName);
-	dump.start(SQLITE_BINARY, QStringList() << m_mainDbPath << ".dump");
-	if (!dump.waitForStarted() || !dump.waitForFinished(-1) || (dump.exitStatus() != QProcess::NormalExit))
-	{
-		QString reason;
-		switch (dump.error())
-		{
-			case (QProcess::FailedToStart) :
-				reason = tr("The process failed to start. Either the invoked program is missing, or you may have insufficient permissions to invoke the program.");
-				break;
-			case (QProcess::Crashed) :
-				reason = tr("The process crashed some time after starting successfully.");
-				break;
-			case (QProcess::WriteError) :
-				reason = tr("An error occurred when attempting to write to the process. For example, the process may not be running, or it may have closed its input channel.");
-				break;
-			case (QProcess::ReadError):
-				reason = tr("An error occurred when attempting to read from the process. For example, the process may not be running.");
-				break;
-			default:
-				reason = tr("An unknown error occurred.");
-		}
-		QMessageBox::warning(this, m_appName, tr("Error creating the dump. Reason: %1").arg(reason));
-	}
+	dump.start(QStringList() << ".dump");
+	if (!dump.success())
+		QMessageBox::warning(this, m_appName, "<qt>" + tr("Error creating the dump. Reason: %1\n%2")
+				.arg(dump.errorMessage()).arg(dump.allStderr()) + "</qt>");
 	else
-		QMessageBox::warning(this, m_appName, tr("Dump written into: %1").arg(fileName));
+	{
+		QString e(dump.allStderr());
+		if (e.isEmpty())
+			QMessageBox::warning(this, m_appName, tr("Dump written into: %1").arg(fileName));
+		else
+			QMessageBox::warning(this, m_appName, tr("An error occured in the dump: %1").arg(e));
+	}
 }
 
 void LiteManWindow::createTable()
@@ -576,17 +682,17 @@ void LiteManWindow::createTable()
 	dlg.exec();
 	if (dlg.update)
 	{
-		foreach (QTreeWidgetItem* item, tableTree->searchMask(tableTree->trTables))
+		foreach (QTreeWidgetItem* item, schemaBrowser->tableTree->searchMask(schemaBrowser->tableTree->trTables))
 		{
 			if (item->type() == TableTree::TablesItemType)
-				tableTree->buildTables(item, item->text(1));
+				schemaBrowser->tableTree->buildTables(item, item->text(1));
 		}
 	}
 }
 
 void LiteManWindow::alterTable()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 
 	if(!item)
 		return;
@@ -594,12 +700,61 @@ void LiteManWindow::alterTable()
 	AlterTableDialog dlg(this, item->text(0), item->text(1));
 	dlg.exec();
 	if (dlg.update)
-		tableTree->buildTables(item->parent(), item->text(1));
+		schemaBrowser->tableTree->buildTables(item->parent(), item->text(1));
+}
+
+void LiteManWindow::renameTable()
+{
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
+	if(!item)
+		return;
+
+	bool ok;
+	QString text = QInputDialog::getText(this, m_appName,
+										 tr("New table name:"), QLineEdit::Normal,
+										 item->text(0), &ok);
+	if (ok && !text.isEmpty())
+	{
+		if (text == item->text(0))
+			return;
+		QString sql = QString("ALTER TABLE \"%1\".\"%2\" RENAME TO \"%3\";")
+								.arg(item->text(1))
+								.arg(item->text(0))
+								.arg(text);
+		if (Database::execSql(sql))
+			schemaBrowser->tableTree->buildTables(item->parent(), item->text(1));
+	}
+}
+
+void LiteManWindow::populateTable()
+{
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
+	if(!item)
+		return;
+	PopulatorDialog dlg(this, item->text(0), item->text(1));
+	dlg.exec();
+	treeItemActivated(item, 0);
+}
+
+void LiteManWindow::importTable()
+{
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
+
+	if(item)
+	{
+		ImportTableDialog dlg(this, item->text(0), item->text(1));
+		dlg.exec();
+	}
+	else
+	{
+		ImportTableDialog dlg(this, "", "main");
+		dlg.exec();
+	}
 }
 
 void LiteManWindow::dropTable()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 
 	if(!item)
 		return;
@@ -611,7 +766,7 @@ void LiteManWindow::dropTable()
 	if(ret == QMessageBox::Yes)
 	{
 		if (Database::dropTable(item->text(0), item->text(1)))
-			tableTree->buildTables(item->parent(), item->text(1));
+			schemaBrowser->tableTree->buildTables(item->parent(), item->text(1));
 	}
 }
 
@@ -621,21 +776,21 @@ void LiteManWindow::createView()
 
 	dia.exec();
 	if(dia.update)
-		tableTree->buildViewTree(dia.schema(), dia.name());
+		schemaBrowser->tableTree->buildViewTree(dia.schema(), dia.name());
 }
 
 void LiteManWindow::alterView()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 	AlterViewDialog dia(item->text(0), item->text(1), this);
 	dia.exec();
 	if (dia.update)
-		tableTree->buildViewTree(item->text(1), item->text(0));
+		schemaBrowser->tableTree->buildViewTree(item->text(1), item->text(0));
 }
 
 void LiteManWindow::dropView()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 
 	if(!item)
 		return;
@@ -648,23 +803,23 @@ void LiteManWindow::dropView()
 	if(ret == QMessageBox::Yes)
 	{
 		if (Database::dropView(item->text(0), item->text(1)))
-			tableTree->buildViews(item->parent(), item->text(1));
+			schemaBrowser->tableTree->buildViews(item->parent(), item->text(1));
 	}
 }
 
 void LiteManWindow::createIndex()
 {
-	QString table(tableTree->currentItem()->parent()->text(0));
-	QString schema(tableTree->currentItem()->parent()->text(1));
+	QString table(schemaBrowser->tableTree->currentItem()->parent()->text(0));
+	QString schema(schemaBrowser->tableTree->currentItem()->parent()->text(1));
 	CreateIndexDialog dia(table, schema, this);
 	dia.exec();
 	if (dia.update)
-		tableTree->buildIndexes(tableTree->currentItem(), schema, table);
+		schemaBrowser->tableTree->buildIndexes(schemaBrowser->tableTree->currentItem(), schema, table);
 }
 
 void LiteManWindow::dropIndex()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 
 	if(!item)
 		return;
@@ -677,7 +832,8 @@ void LiteManWindow::dropIndex()
 	if(ret == QMessageBox::Yes)
 	{
 		if (Database::dropIndex(item->text(0), item->text(1)))
-			tableTree->buildIndexes(item->parent(), item->text(1), item->text(0));
+			schemaBrowser->tableTree->buildIndexes(item->parent(), item->text(1),
+												   item->parent()->parent()->text(0));
 	}
 }
 
@@ -687,44 +843,34 @@ void LiteManWindow::treeItemActivated(QTreeWidgetItem * item, int /*column*/)
 		return;
 
 	if (item->type() == TableTree::TableType || item->type() == TableTree::ViewType
-	    || item->type() == TableTree::SystemType)
+		|| item->type() == TableTree::SystemType)
 	{
-// 		sqlEditor->setVisible(false);
-		dataViewer->setEnabled(true);
-// 		dataViewer->showStatusText(false);
-
+		dataViewer->freeResources();
 		if(item->type() == TableTree::ViewType || item->type() == TableTree::SystemType)
 		{
-			dataViewer->showButtons(false);
-			SqlQueryModel * model = new SqlQueryModel(this);
+			SqlQueryModel * model = new SqlQueryModel(0);
 			model->setQuery(QString("select * from %1.%2").arg(item->text(1)).arg(item->text(0)), QSqlDatabase::database(SESSION_NAME));
-			dataViewer->setTableModel(model);
+			dataViewer->setTableModel(model, false);
 		}
 		else
 		{
-			dataViewer->showButtons(true);
-			SqlTableModel * model = new SqlTableModel(this, QSqlDatabase::database(attachedDb[item->text(1)]));
+			SqlTableModel * model = new SqlTableModel(0, QSqlDatabase::database(attachedDb[item->text(1)]));
 			model->setSchema(item->text(1));
 			model->setTable(item->text(0));
 			model->select();
 			model->setEditStrategy(SqlTableModel::OnManualSubmit);
-			dataViewer->setTableModel(model);
+			dataViewer->setTableModel(model, true);
 		}
 	}
 }
 
-void LiteManWindow::tableTree_currentItemChanged(QTreeWidgetItem* cur, QTreeWidgetItem* prev)
-{
-	buildContextMenu(cur);
-}
-
-void LiteManWindow::buildContextMenu(QTreeWidgetItem * item)
+void LiteManWindow::tableTree_currentItemChanged(QTreeWidgetItem* cur, QTreeWidgetItem* /*prev*/)
 {
 	contextMenu->clear();
-	if (!item)
+	if (!cur)
 		return;
 
-	switch(item->type())
+	switch(cur->type())
 	{
 		case TableTree::TablesItemType:
 			contextMenu->addAction(createTableAct);
@@ -737,8 +883,12 @@ void LiteManWindow::buildContextMenu(QTreeWidgetItem * item)
 		case TableTree::TableType:
 			contextMenu->addAction(describeTableAct);
 			contextMenu->addAction(alterTableAct);
+			contextMenu->addAction(renameTableAct);
 			contextMenu->addAction(dropTableAct);
 			contextMenu->addAction(reindexAct);
+			contextMenu->addSeparator();
+			contextMenu->addAction(importTableAct);
+			contextMenu->addAction(populateTableAct);
 			break;
 
 		case TableTree::ViewType:
@@ -759,13 +909,14 @@ void LiteManWindow::buildContextMenu(QTreeWidgetItem * item)
 
 		case TableTree::DatabaseItemType:
 			contextMenu->addAction(refreshTreeAct);
-			if (item->text(0) != "main")
+			if (cur->text(0) != "main")
 				contextMenu->addAction(detachAct);
 			break;
 
 		case TableTree::TriggersItemType:
 			contextMenu->addAction(createTriggerAct);
-			contextMenu->addAction(consTriggAct);
+			if (cur->parent()->type() != TableTree::ViewType)
+				contextMenu->addAction(consTriggAct);
 			break;
 
 		case TableTree::TriggerType:
@@ -781,52 +932,21 @@ void LiteManWindow::buildContextMenu(QTreeWidgetItem * item)
 void LiteManWindow::treeContextMenuOpened(const QPoint & pos)
 {
 	if (contextMenu->actions().count() != 0)
-		contextMenu->exec(tableTree->viewport()->mapToGlobal(pos));
+		contextMenu->exec(schemaBrowser->tableTree->viewport()->mapToGlobal(pos));
 }
 
-void LiteManWindow::describeTable()
+void LiteManWindow::describeObject()
 {
-	runQuery(QString("pragma \"%1\".table_info (\"%2\");")
-			.arg(tableTree->currentItem()->text(1))
-			.arg(tableTree->currentItem()->text(0)));
-}
-
-void LiteManWindow::describeView()
-{
-	runQuery(QString("select sql from \"%1\".sqlite_master where name = '%2';")
-			.arg(tableTree->currentItem()->text(1))
-			.arg(tableTree->currentItem()->text(0)));
-}
-
-void LiteManWindow::describeIndex()
-{
-	runQuery(QString("pragma \"%1\".index_info (\"%2\");")
-			.arg(tableTree->currentItem()->text(1))
-			.arg(tableTree->currentItem()->text(0)));
-}
-
-void LiteManWindow::describeTrigger()
-{
-	runQuery(QString("select sql from \"%1\".sqlite_master where name = '%2';")
-			.arg(tableTree->currentItem()->text(1))
-			.arg(tableTree->currentItem()->text(0)));
-}
-
-void LiteManWindow::runQuery(QString statement)
-{
-	SqlQueryModel * model = new SqlQueryModel(this);
-	model->setQuery(statement, QSqlDatabase::database(SESSION_NAME));
-	if (!dataViewer->setTableModel(model))
-		return;
-
-	dataViewer->setEnabled(true);
-	dataViewer->showStatusText(true);
-	dataViewer->showButtons(false);
-
-	if(model->lastError().isValid())
-		dataViewer->setStatusText(tr("Query Error: %1").arg(model->lastError().databaseText()));
-	else
-		dataViewer->setStatusText(tr("Query OK\nRow(s) returned: %1").arg(model->rowCount()));
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
+	/*runQuery*/
+// 	execSql(QString("select sql as \"%1\" from \"%2\".sqlite_master where name = '%3';")
+// 			.arg(tr("Describe %1").arg(item->text(0).toUpper()))
+// 			.arg(item->text(1))
+// 			.arg(item->text(0)));
+	QString desc(Database::describeObject(item->text(0), item->text(1)));
+	dataViewer->sqlScriptStart();
+	dataViewer->showSqlScriptResult("-- " + tr("Describe %1").arg(item->text(0).toUpper()));
+	dataViewer->showSqlScriptResult(desc);
 }
 
 void LiteManWindow::analyzeDialog()
@@ -834,10 +954,10 @@ void LiteManWindow::analyzeDialog()
 	AnalyzeDialog *dia = new AnalyzeDialog(this);
 	dia->exec();
 	delete dia;
-	foreach (QTreeWidgetItem* item, tableTree->searchMask(tableTree->trSys))
+	foreach (QTreeWidgetItem* item, schemaBrowser->tableTree->searchMask(schemaBrowser->tableTree->trSys))
 	{
 		if (item->type() == TableTree::SystemItemType)
-			tableTree->buildCatalogue(item, item->text(1));
+			schemaBrowser->tableTree->buildCatalogue(item, item->text(1));
 	}
 }
 
@@ -857,8 +977,8 @@ void LiteManWindow::attachDatabase()
 	bool ok;
 	QFileInfo f(fileName);
 	QString schema = QInputDialog::getText(this, tr("Attach Database"),
-                                          tr("Enter a Schema Alias:"), QLineEdit::Normal,
-                                          f.baseName(), &ok);
+										  tr("Enter a Schema Alias:"), QLineEdit::Normal,
+										  f.baseName(), &ok);
 	if (!ok || schema.isEmpty())
 		return;
 	if (!Database::execSql(QString("attach database '%1' as \"%2\";").arg(fileName).arg(schema)))
@@ -874,12 +994,12 @@ void LiteManWindow::attachDatabase()
 		return;
 	}
 
-	tableTree->buildDatabase(schema);
+	schemaBrowser->tableTree->buildDatabase(schema);
 }
 
 void LiteManWindow::detachDatabase()
 {
-	QString dbname(tableTree->currentItem()->text(0));
+	QString dbname(schemaBrowser->tableTree->currentItem()->text(0));
 	if (!Database::execSql(QString("detach database \"%1\";").arg(dbname)))
 		return;
 
@@ -887,24 +1007,26 @@ void LiteManWindow::detachDatabase()
 	QSqlDatabase::database(attachedDb[dbname]).close();
 	attachedDb.remove(dbname);
 
-	delete tableTree->currentItem();
+	delete schemaBrowser->tableTree->currentItem();
 }
 
 void LiteManWindow::createTrigger()
 {
-	QString table(tableTree->currentItem()->parent()->text(0));
-	QString schema(tableTree->currentItem()->parent()->text(1));
-	CreateTriggerDialog *dia = new CreateTriggerDialog(table, schema, this);
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
+	QString table(item->parent()->text(0));
+	QString schema(item->parent()->text(1));
+	CreateTriggerDialog *dia = new CreateTriggerDialog(table, schema,
+											item->parent()->type(), this);
 	dia->exec();
 	if (dia->update)
-		tableTree->buildTriggers(tableTree->currentItem(), schema, table);
+		schemaBrowser->tableTree->buildTriggers(item, schema, table);
 	delete dia;
 }
 
 void LiteManWindow::alterTrigger()
 {
-	QString table(tableTree->currentItem()->text(0));
-	QString schema(tableTree->currentItem()->text(1));
+	QString table(schemaBrowser->tableTree->currentItem()->text(0));
+	QString schema(schemaBrowser->tableTree->currentItem()->text(1));
 	AlterTriggerDialog *dia = new AlterTriggerDialog(table, schema, this);
 	dia->exec();
 	delete dia;
@@ -913,7 +1035,7 @@ void LiteManWindow::alterTrigger()
 
 void LiteManWindow::dropTrigger()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 
 	if(!item)
 		return;
@@ -926,23 +1048,23 @@ void LiteManWindow::dropTrigger()
 	if(ret == QMessageBox::Yes)
 	{
 		if (Database::dropTrigger(item->text(0), item->text(1)))
-			tableTree->buildTriggers(item->parent(), item->text(1), item->parent()->parent()->text(0));
+			schemaBrowser->tableTree->buildTriggers(item->parent(), item->text(1), item->parent()->parent()->text(0));
 	}
 }
 
 void LiteManWindow::constraintTriggers()
 {
-	QString table(tableTree->currentItem()->parent()->text(0));
-	QString schema(tableTree->currentItem()->parent()->text(1));
+	QString table(schemaBrowser->tableTree->currentItem()->parent()->text(0));
+	QString schema(schemaBrowser->tableTree->currentItem()->parent()->text(1));
 	ConstraintsDialog dia(table, schema, this);
 	dia.exec();
 	if (dia.update)
-		tableTree->buildTriggers(tableTree->currentItem(), schema, table);
+		schemaBrowser->tableTree->buildTriggers(schemaBrowser->tableTree->currentItem(), schema, table);
 }
 
 void LiteManWindow::reindex()
 {
-	QTreeWidgetItem * item = tableTree->currentItem();
+	QTreeWidgetItem * item = schemaBrowser->tableTree->currentItem();
 	if(!item)
 		return;
 	QString sql(QString("REINDEX \"%1\".\"%2\";").arg(item->text(1)).arg(item->text(0)));
@@ -953,5 +1075,6 @@ void LiteManWindow::preferences()
 {
 	PreferencesDialog prefs(this);
 	if (prefs.exec())
-		prefs.saveSettings();
+		if (prefs.saveSettings())
+			emit prefsChanged();
 }
